
import java.io.*;
import java.util.*;

public class CountCheck extends SentenceFilter {

    public static final int[][] counts = new int[2256][5];
    public PrintWriter log;
    public boolean unknowns;

    public CountCheck() {
	readArray("counts", counts);
	try {
	   log = new PrintWriter(new FileWriter("countcheck.log"),true);
       } catch (IOException e) {
	   System.err.println(e);
	   System.exit(1);
       }

    }

    public void finalize() {
	log.close();
    }

    public int[] resultFormula(String[] stags) {
	
	int[] totalcounts = new int[] {0, 0, 0, 0, 0};

	unknowns = false;

	for (int i = 0; i < stags.length; i++) {
	    int st = Integer.parseInt(stags[i]);
	    if (st == 0) {
		unknowns = true;
	    }
	    for (int j = 0; j < 5; j++) {
		totalcounts[j] = totalcounts[j] + counts[st][j];
	    }
	}

	return totalcounts;
    }

    public int[] resultFormula(String stags) {
	
	int[] totalcounts = new int[] {0, 0, 0, 0, 0};

	StringTokenizer st = new StringTokenizer(stags);
	int x;
	unknowns = false;

	while(st.hasMoreTokens()) {
	    x = Integer.parseInt(st.nextToken());
	    if (x == 0) {
		unknowns = true;
	    }
	    for (int j = 0; j < 5; j++) {
		totalcounts[j] = totalcounts[j] + counts[x][j];
	    }
	}

	return totalcounts;
    }
  

   protected boolean consistentSentence(String[] stags) {

       return countCheck(resultFormula(stags));

    }

   public boolean countCheck(int[] totalcounts) {

       log.println("...");
       printCounts(log, totalcounts);
       return (((totalcounts[0]==1 && totalcounts[1]==0 && totalcounts[3]==0) ||
	       (totalcounts[0]==1 && totalcounts[1]==-1 && totalcounts[3]==0) ||
	       (totalcounts[0]==0 && totalcounts[1]==1 && totalcounts[3]==0) ||
	       (totalcounts[0]==0 && totalcounts[1]==0 && totalcounts[3]==1) ||
	       (totalcounts[0]==0 && totalcounts[1]==-1 && totalcounts[3]==1) ||
		(totalcounts[0]==0 && totalcounts[1]==0 && totalcounts[3]==0))
	       && totalcounts[2]==0 && totalcounts[4]==0);

    }


    public void readArray(String n, int[][] tmp) {

	tmp[0][0] = 0;
	tmp[0][1] = 0;
	tmp[0][2] = 0;
	tmp[0][3] = 0;
	tmp[0][4] = 0;

	try {
	    File file = new File(n);
	    BufferedReader r = new BufferedReader(new FileReader(file));
	    int i = 1;
	    int j = 0;
	    String line = null;
	    while ((line=r.readLine()) != null) {
		j = 0;
		StringTokenizer st = new StringTokenizer(line);
		while(st.hasMoreTokens()) {
		    tmp[i][j] = Integer.parseInt(st.nextToken());
		    j++;
		}
		i++;
	    }
 	} catch (IOException e) {
	    System.err.println(e);
	    System.exit(1);
	}
    }

    public String getResults(String ts) {

	StringBuffer sb = new StringBuffer();
	StringTokenizer st = new StringTokenizer(ts);
	String s = null;
	int i;

	while(st.hasMoreTokens()) {
	    s = st.nextToken();
	    i = s.lastIndexOf("/");
	    sb.append(s.substring(i+1)+" ");
	}

	return sb.toString().trim();
    }


    public void printCounts(PrintWriter outf, int[] c) {
	  outf.println("s   = "+c[0]);
	  outf.println("np  = "+c[1]);
	  outf.println("n   = "+c[2]);
	  outf.println("pp  = "+c[3]);
	  outf.println("let = "+c[4]); 
    }

    public static void main(String[] args) {
	  
	  CountCheck c = new CountCheck();
	  int[] f = new int[] {0, 0, 0, 0, 0};
	  String line, results;

	  int i = 0;

	  try {

		BufferedReader inf = new BufferedReader(new FileReader(args[0]));
		PrintWriter outf = new PrintWriter(new FileWriter(args[1]),true);

		while ((line=inf.readLine()) != null) {
		    i++;
		    outf.println(".. "+i+" ..");
		    results = c.getResults(line);
		    f = c.resultFormula(results);
		    c.printCounts(outf, f);
		    outf.println(c.countCheck(f));
		    if (!c.countCheck(f)) {
			System.out.println(i + ":" + line);
		    }
		}
	  } catch (Exception e) {
	      System.err.println(e);
	      System.exit(1);
	  }
	  
    }
}