
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import java.io.*;
import java.util.*;

import org.jdom.*;

import opennlp.common.preprocess.*;
import opennlp.common.xml.*;
import opennlp.common.util.*;

import opennlp.maxent.*;
import opennlp.maxent.io.*;

public class EvaluationGUI extends JPanel {
    public final static int ONE_SECOND = 1000;

    private static JProgressBar progressBar = new JProgressBar(0, 1000);
    //private static JPanel graph;

    public static String removeResults(String ts) {


	StringBuffer sb = new StringBuffer();
	StringTokenizer st = new StringTokenizer(ts);
	String s = null;
	int i;

	while(st.hasMoreTokens()) {
	    s = st.nextToken();
	    i = s.lastIndexOf("/");
	    sb.append(s.substring(0,i)+" ");
	}

	return sb.toString().trim();
    }

    public EvaluationGUI() {
        super(new BorderLayout());

	progressBar.setValue(0);

        JPanel panel = new JPanel(new GridLayout(0,1));
        panel.add(progressBar);

        add(panel, BorderLayout.PAGE_END);
        setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));

    }

    private static void createAndShowGUI() {

	JMenuBar menuBar;
	JMenu fileMenu, helpMenu;
	JMenuItem load, quit, help;

        //Make sure we have nice window decorations.
        JFrame.setDefaultLookAndFeelDecorated(true);

        //Create and set up the window.
        JFrame frame = new JFrame("Evaluation");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        //Create and set up the content pane.
        JComponent newContentPane = new EvaluationGUI();
        newContentPane.setOpaque(true); //content panes must be opaque
        frame.setContentPane(newContentPane);


	menuBar = new JMenuBar();
	menuBar.setLayout(new BorderLayout());
	frame.setJMenuBar(menuBar);

	fileMenu = new JMenu("File");
	fileMenu.setMnemonic(KeyEvent.VK_F);

	load = new JMenuItem("Load...", KeyEvent.VK_L);
	fileMenu.add(load);

	quit = new JMenuItem("Quit", KeyEvent.VK_Q);
	fileMenu.add(quit);

	helpMenu = new JMenu("Help");
	helpMenu.setMnemonic(KeyEvent.VK_H);

	help = new JMenuItem("About this program...", KeyEvent.VK_A);
	helpMenu.add(help);

	menuBar.add(fileMenu, BorderLayout.LINE_START);
	menuBar.add(helpMenu, BorderLayout.LINE_END);

        //Display the window.
        frame.pack();
        frame.setVisible(true);
    }

    public static void main(String[] args) {

	try {
	    if (args.length >= 3) {
		//Schedule a job for the event-dispatching thread:
		//creating and showing this application's GUI.
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
			    createAndShowGUI();
			}
		    });

		progressBar.setIndeterminate(true);

		System.out.print("Reading model...");
                SuperTaggerME st = new SuperTaggerME(new SuffixSensitiveGISModelReader(new File(args[2])).getModel());
		System.out.println("done!");

		// count the number of lines
		BufferedReader tmp = new BufferedReader(new FileReader(args[0]));
		int totallines = 0;
		while (tmp.readLine() != null) {
		    totallines++;
		}
		tmp.close();

		progressBar.setIndeterminate(false);


		BufferedReader inf = new BufferedReader(new FileReader(args[0]));
		PrintWriter outf = new PrintWriter(new FileWriter(args[1]));

		int lines = 0;
		String line = null;

		while ((line=inf.readLine()) != null) {
		    line = removeResults(line);
		    outf.println(st.supertag(line));
		    System.out.print(".");
		    lines++;
		    progressBar.setValue((lines*1000)/totallines);
		}

		System.out.println();
		outf.close();
		inf.close();

		System.exit(0);

	    } else {
		System.out.println("Usage: java EvaluationME infile outfile model");
	    }
	} catch (IOException e) {
	    System.err.println(e);
	    System.exit(1);
	}
    }
}
