///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2002 Jason Baldridge and Gann Bierner
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////////////////////////////////////////////////////////////////////////

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import opennlp.maxent.Evalable;
import opennlp.maxent.EventCollector;
import opennlp.maxent.EventStream;
import opennlp.maxent.GISModel;
import opennlp.maxent.MaxentModel;
import opennlp.maxent.TrainEval;
import opennlp.maxent.PlainTextByLineDataStream;
import opennlp.maxent.TwoPassDataIndexer;
import opennlp.maxent.EventCollectorAsStream;
import opennlp.maxent.io.SuffixSensitiveGISModelReader;
import opennlp.maxent.io.SuffixSensitiveGISModelWriter;
import opennlp.tools.util.Sequence;

/**
 * A supertagger that uses maximum entropy. Straightforward adaptation
 * of the part-of-speech tagger by Gann Bierner.
 */
public class SuperTaggerME extends TriSuperTaggerME implements Evalable, SuperTagger {

    public SuperTaggerME(MaxentModel mod) {
	super(DEFAULT_BEAM_SIZE, mod, new STContextGenerator());
    }
  
    public SuperTaggerME(MaxentModel mod, STContextGenerator cg) {
	super(DEFAULT_BEAM_SIZE, mod, cg);
    }

    public SuperTaggerME(int beamSize, MaxentModel mod, STContextGenerator cg) {
	super(beamSize, mod, cg);
    }
    
  /**
     * <p>Trains a new pos model.</p>
     *
     * <p>Usage: java opennlp.postag.SuperTaggerME data_file new_model_name (iterations cutoff)?</p>
     *
     */

  public static void main(String[] args) throws IOException {
    if (args.length == 0){
      System.err.println("Usage: SuperTaggerME training model iterations cutoff");
      System.err.println("This trains a new model on the specified training file and writes the trained model to the model file.");
      System.exit(1);
    }
    try {
      File inFile = new File(args[0]);
      File outFile = new File(args[1]);
      GISModel mod;

      EventStream es = new EventCollectorAsStream(new STEventCollector(new FileReader(inFile), new STContextGenerator()));
      if (args.length > 3)
        mod = train(es, Integer.parseInt(args[2]), Integer.parseInt(args[3]));
      else
        mod = train(es, 100, 5);

      System.out.println("Saving the model as: " + args[1]);
      new SuffixSensitiveGISModelWriter(mod, outFile).persist();

    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}
