import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import opennlp.maxent.Evalable;
import opennlp.maxent.EventCollector;
import opennlp.maxent.EventStream;
import opennlp.maxent.GISModel;
import opennlp.maxent.MaxentModel;
import opennlp.maxent.TrainEval;
import opennlp.maxent.PlainTextByLineDataStream;
import opennlp.maxent.TwoPassDataIndexer;
import opennlp.maxent.EventCollectorAsStream;
import opennlp.maxent.io.SuffixSensitiveGISModelReader;
import opennlp.maxent.io.SuffixSensitiveGISModelWriter;
import opennlp.tools.util.Sequence;

public class TriSuperTaggerME extends AbstractSuperTaggerME implements Evalable, SuperTagger {

  /**
     * <p>Trains a new pos model.</p>
     *
     * <p>Usage: java opennlp.postag.TriSuperTaggerME data_file new_model_name (iterations cutoff)?</p>
     *
     */

    protected TriSuperTaggerME(MaxentModel mod) {
	super(DEFAULT_BEAM_SIZE, mod, new TriContextGenerator());
    }
  
    protected TriSuperTaggerME(MaxentModel mod, TriContextGenerator cg) {
	super(DEFAULT_BEAM_SIZE, mod, cg);
    }

    protected TriSuperTaggerME(int beamSize, MaxentModel mod, TriContextGenerator cg) {
	super(beamSize, mod, cg);
    }

    public static void main(String[] args) throws IOException {
	if (args.length == 0){
	    System.err.println("Usage: TriSuperTaggerME training model iterations cutoff");
	    System.err.println("This trains a new model on the specified training file and writes the trained model to the model file.");
	    System.exit(1);
	}
	try {
	    File inFile = new File(args[0]);
	    File outFile = new File(args[1]);
	    GISModel mod;
	    
	    EventStream es = new EventCollectorAsStream(new STEventCollector(new FileReader(inFile), new TriContextGenerator()));
	    if (args.length > 3)
		mod = train(es, Integer.parseInt(args[2]), Integer.parseInt(args[3]));
	    else
		mod = train(es, 100, 5);
	    
	    System.out.println("Saving the model as: " + args[1]);
	    new SuffixSensitiveGISModelWriter(mod, outFile).persist();
	    
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
    
}